from apis_core.apis_entities.models import Person
from django.contrib.auth.models import User
from django.db import models
from django.db.models import DEFERRED
from django.forms.models import model_to_dict
from django.contrib import admin


CHOICES_STAGE = (
    ("ausgeschrieben", "ausgeschrieben"),
    ("autordurchsicht", "autordurchsicht"),
    ("eingelangt", "eingelangt"),
    ("endredaktion", "endredaktion"),
    ("gruppenredaktion", "gruppenredaktion"),
    ("verteilt", "verteilt"),
    ("zugesagt", "zugesagt"),
)

CHOICES_BIOUSER = (("aut", "AutorIn"), ("red", "RedakteurIn"), ("kor", "Korrektorat"))


class Contract(models.Model):
    contract = models.FilePathField()
    issued = models.DateField()
    lemma = models.ForeignKey("Lemma", on_delete=models.CASCADE)
    author = models.ForeignKey("Author", on_delete=models.CASCADE)


class IrsUser(User):
    """Model to inherit from Django User model"""

    pass


class Editor(IrsUser):
    """Model for members of the editorial team"""

    pass


class Author(IrsUser):
    """Model for authors of biographies"""

    address = models.TextField(null=True, blank=True)


class Issue(models.Model):
    """Model holding Issues. Issues are a collection of Bios that will be published
    at a certain date. Pre digital-only, that was a volume.
    """

    name = models.CharField(max_length=255)
    pubDate = models.DateField(verbose_name="Publication Date", blank=True, null=True)


class LemmaNote(models.Model):
    """Model holding information on Notes that have been created by users on a Lemma."""

    created = models.DateTimeField(auto_now=True)
    text = models.TextField()
    user = models.ForeignKey("IrsUser", on_delete=models.CASCADE)
    lemma = models.ForeignKey("Lemma", on_delete=models.CASCADE)


class LemmaLabel(models.Model):
    """Model holding labels attached to Lemmas"""

    name = models.CharField(max_length=255)


class LemmaStatus(models.Model):
    """Model holding the Status of the Lemma"""

    name = models.CharField(max_length=255)


class Lemma(Person):
    """Model holding the information on the Lemma. This class is inheritating the APIS
    Person class
    """

    pass


class IssueLemma(models.Model):
    """Connects the lemma to all the other information. Rather than modifying the object
    a new one is created on all data changes. This allows for a complete revision history.
    """

    created = models.DateTimeField(auto_now=True)
    issue = models.ForeignKey("Issue", on_delete=models.SET_NULL, null=True)
    status = models.ForeignKey("LemmaStatus", on_delete=models.SET_NULL, null=True)
    lemma = models.ForeignKey("Lemma", on_delete=models.SET_NULL, null=True)
    author = models.ForeignKey("Author", on_delete=models.SET_NULL, null=True)
    editor = models.ForeignKey("Editor", on_delete=models.SET_NULL, null=True)
    serialization = models.JSONField(null=True, blank=True)

    @classmethod
    def from_db(cls, db, field_names, values):
        # Default implementation of from_db() (subject to change and could
        # be replaced with super()).
        if len(values) != len(cls._meta.concrete_fields):
            values = list(values)
            values.reverse()
            values = [
                values.pop() if f.attname in field_names else DEFERRED
                for f in cls._meta.concrete_fields
            ]
        instance = cls(*values)
        instance._state.adding = False
        instance._state.db = db
        # customization to store the original field values on the instance
        instance._loaded_values = dict(zip(field_names, values))
        return instance

    def save(self, *args, **kwargs):
        if not self._state.adding:
            created = self._loaded_values["created"]
            if self._loaded_values["issue_id"] is not None:
                issue = model_to_dict(
                    Issue.objects.get(pk=self._loaded_values["issue_id"])
                )
                issue["pubDate"] = issue["pubDate"].isoformat()
            else:
                issue = None
            if self._loaded_values["status_id"] is not None:
                status = model_to_dict(
                    LemmaStatus.objects.get(pk=self._loaded_values["status_id"])
                )
            else:
                status = None
            if self._loaded_values["lemma_id"] is not None:
                lemma = model_to_dict(
                    Lemma.objects.get(pk=self._loaded_values["lemma_id"])
                )
            else:
                lemma = None
            if self._loaded_values["author_id"] is not None:
                author = model_to_dict(
                    Author.objects.get(pk=self._loaded_values["author_id"])
                )
            else:
                author = None
            if self._loaded_values["editor_id"] is not None:
                editor = model_to_dict(
                    Editor.objects.get(pk=self._loaded_values["editor_id"])
                )
            else:
                editor = None
            res = {
                "created": created.isoformat(),
                "issue": issue,
                "status": status,
                "lemma": lemma,
                "author": author,
                "editor": editor,
            }
            if self.serialization is not None:
                self.serialization.append(res)
            else:
                self.serialization = [
                    res,
                ]
            super().save(*args, **kwargs)
        else:
            super().save(*args, **kwargs)



from rest_framework import filters, viewsets, renderers
from rest_framework.permissions import IsAuthenticated
from django_filters.rest_framework import DjangoFilterBackend
from .models import Author, Issue, IssueLemma, LemmaStatus, LemmaNote, Lemma
from .serializers import (
    AuthorSerializer,
    EditorSerializer,
    IssueSerializer,
    IssueLemmaSerializer,
    LemmaStatusSerializer,
    LemmaNoteSerializer,
    LemmaSerializer,
)


class AuthorViewset(viewsets.ReadOnlyModelViewSet):
    """Viewset to retrieve Author objects"""

    queryset = Author.objects.all()
    serializer_class = AuthorSerializer
    filterset_fields = ["username"]
    permission_classes = [IsAuthenticated]


class IssueViewset(viewsets.ModelViewSet):

    queryset = Issue.objects.all()
    serializer_class = IssueSerializer
    filter_fields = ["name", "pubDate"]
    permission_classes = [IsAuthenticated]


class IssueLemmaViewset(viewsets.ModelViewSet):

    queryset = IssueLemma.objects.all()
    serializer_class = IssueLemmaSerializer
    filter_fields = ["lemma", "issue", "author", "editor"]
    permission_classes = [IsAuthenticated]


class LemmaViewset(viewsets.ReadOnlyModelViewSet):

    queryset = Lemma.objects.all()
    serializer_class = LemmaSerializer
    permission_classes = [IsAuthenticated]


class LemmaNoteViewset(viewsets.ModelViewSet):

    queryset = LemmaNote.objects.all()
    serializer_class = LemmaNoteSerializer
    filter_fields = ["lemma"]
    permission_classes = [IsAuthenticated]


class LemmaStatusViewset(viewsets.ModelViewSet):

    queryset = LemmaStatus.objects.all()
    serializer_class = LemmaStatusSerializer
    filter_fields = ["issuelemma"]
    permission_classes = [IsAuthenticated]


# Generated by Django 3.1.4 on 2020-12-11 19:20

import django.contrib.auth.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('apis_entities', '0004_auto_20200722_1231'),
    ]

    operations = [
        migrations.CreateModel(
            name='IrsUser',
            fields=[
                ('user_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='auth.user')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            bases=('auth.user',),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Issue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('pubDate', models.DateField(blank=True, null=True, verbose_name='Publication Date')),
            ],
        ),
        migrations.CreateModel(
            name='Lemma',
            fields=[
                ('person_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='apis_entities.person')),
            ],
            options={
                'abstract': False,
            },
            bases=('apis_entities.person',),
        ),
        migrations.CreateModel(
            name='LemmaLabel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='LemmaStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Author',
            fields=[
                ('irsuser_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='oebl_irs_workflow.irsuser')),
                ('address', models.TextField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            bases=('oebl_irs_workflow.irsuser',),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Editor',
            fields=[
                ('irsuser_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='oebl_irs_workflow.irsuser')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'abstract': False,
            },
            bases=('oebl_irs_workflow.irsuser',),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='LemmaNote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now=True)),
                ('text', models.TextField()),
                ('lemma', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='oebl_irs_workflow.lemma')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='oebl_irs_workflow.irsuser')),
            ],
        ),
        migrations.CreateModel(
            name='IssueLemma',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now=True)),
                ('serialization', models.JSONField(blank=True, null=True)),
                ('issue', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='oebl_irs_workflow.issue')),
                ('lemma', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='oebl_irs_workflow.lemma')),
                ('status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='oebl_irs_workflow.lemmastatus')),
                ('author', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='oebl_irs_workflow.author')),
                ('editor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='oebl_irs_workflow.editor')),
            ],
        ),
        migrations.CreateModel(
            name='Contract',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contract', models.FilePathField()),
                ('issued', models.DateField()),
                ('lemma', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='oebl_irs_workflow.lemma')),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='oebl_irs_workflow.author')),
            ],
        ),
    ]



from rest_framework import serializers
from .models import Author, Editor, Issue, IssueLemma, LemmaStatus, Lemma, LemmaNote


class UserSerializer(serializers.ModelSerializer):
    userId = serializers.IntegerField(source="pk")
    name = serializers.SerializerMethodField(method_name="get_name")

    def get_name(self, object):
        if object.last_name is not None:
            return f"{object.last_name}, {object.first_name}"
        else:
            return object.username

    class Meta:
        model = Author
        fields = ["userId", "email", "name"]


class AuthorSerializer(UserSerializer):
    class Meta:
        model = Author
        fields = ["userId", "email", "name", "address"]


class EditorSerializer(UserSerializer):
    class Meta:
        model = Editor
        fields = ["userId", "email", "name"]


class LemmaSerializer(serializers.ModelSerializer):
    class Meta:
        model = Lemma
        fields = "__all__"


class LemmaStatusSerializer(serializers.ModelSerializer):
    class Meta:
        model = LemmaStatus
        fields = "__all__"


class IssueSerializer(serializers.ModelSerializer):
    class Meta:
        model = Issue
        fields = "__all__"


class LemmaNoteSerializer(serializers.ModelSerializer):
    class Meta:
        model = LemmaNote
        fields = "__all__"


class IssueLemmaSerializer(serializers.ModelSerializer):
    notes = serializers.SerializerMethodField(method_name="get_notes")

    def get_notes(self, object):
        res = LemmaNote.objects.filter(lemma=object.lemma)
        return LemmaNoteSerializer(res, many=True).data

    class Meta:
        model = IssueLemma
        fields = "__all__"
        read_only_fields = ["serialization"]
        depth = 1


from rest_framework import routers
from .api_views import (
    AuthorViewset,
    IssueViewset,
    IssueLemmaViewset,
    LemmaStatusViewset,
    LemmaNoteViewset,
    LemmaViewset,
)

app_name = "oebl_irs_workflow"

router = routers.DefaultRouter()
router.register(r"authors", AuthorViewset)
router.register(r"issue-lemma", IssueLemmaViewset)
router.register(r"issues", IssueViewset)
router.register(r"lemma", LemmaViewset)
router.register(r"lemma-status", LemmaStatusViewset)
router.register(r"lemma-note", LemmaNoteViewset)

urlpatterns = router.urls



from django.contrib import admin
from .models import Author, Editor, Lemma, Issue, IssueLemma, LemmaStatus, LemmaNote

admin.site.register(Author)
admin.site.register(Editor)
admin.site.register(Lemma)
admin.site.register(Issue)
admin.site.register(IssueLemma)
admin.site.register(LemmaStatus)
admin.site.register(LemmaNote)


"""
Json Schema to Django Model
"""
import json
import argparse
import logging
import os


def determine_model_name(model_id=None, filename=None):
    """
    Get the model name
    :param model_id: str, model id
    :param filename: str, filename
    :return: str, model name
    """
    model_name = ""
    if model_id:
        try:
            model_name = model_id.split("/")[-1].replace(".json", "")
        except Exception as e:
            logging.exception("Unhandled exception {}".format(e))

    if not model_name and filename:
        filename = filename.strip(os.sep)
        model_name = filename.split(os.sep)[-1]
        model_name = model_name.split(".")[0]

    return model_name.capitalize() or "UnknownModel"


def get_required_string(
    key_name, required_fields, field_type="string", is_pk_field=False
):
    """
    Gets the required portion of model field
    :param key_name:
    :param required_fields:
    :return: str, required model string
    """
    if is_pk_field:
        return "primary_key=True"

    if field_type in ("number", "array", "object"):
        if key_name in required_fields:
            return "null=False"
        return "null=True"
    else:  # string, boolean
        if key_name in required_fields:
            return "null=False, blank=False"
        return "null=True, blank=True"


def parse_model():
    parser = argparse.ArgumentParser()
    parser.add_argument("filename")
    args = parser.parse_args()
    filename = args.filename

    with open(filename) as f:
        json_model = json.load(f)

    # Make sure not list, but object
    if json_model["type"] != "object":
        print(
            "Model type has to be object to convert to model, got {}".format(
                json_model["type"]
            )
        )

    if "oneOf" in json_model:
        print("Optional required fields detected: {}".format(json_model["oneOf"]))

    # Default model string
    model_str = "\nfrom djongo import models\nfrom djongo.models import json\n\n"

    model_name = determine_model_name(json_model.get("id"), args.filename)
    model_str += "class {}(models.Model):\n".format(model_name)
    model_str += '    """Generated model from json schema"""\n'
    print("Model name is {}".format(model_name))

    if "title" in json_model:
        print("Title of model is {}".format(json_model["title"]))

    if "description" in json_model:
        print("Description of model is {}".format(json_model["description"]))

    required_fields = []
    if "required" in json_model:
        required_fields = json_model["required"]

    for key_name, key_attributes in json_model["properties"].items():
        if key_name.endswith("_id") and key_name != "_id":
            print("WARNING: Possible ForeignKey {}".format(key_name))

        # if key_attributes['type'] == 'array':
        #     print("ERROR: Unsupported type array, skipping for field {}".format(key_name))

        if key_attributes["type"] == "null":
            print(
                "ERROR: Unsupported type null, skipping for field {}".format(key_name)
            )

        # PK field
        is_pk_field = False
        if key_name in ["id", "_id"]:
            is_pk_field = True

        # If required field
        required_str = get_required_string(
            key_name, required_fields, key_attributes["type"], is_pk_field
        )
        field_str = ""

        # String choice field, enum
        if key_attributes["type"] == "string" and "enum" in key_attributes:
            if not key_attributes["enum"]:
                print(
                    "ERROR: Missing enum for enum choice field {}, skipping..".format(
                        key_name
                    )
                )
                continue

            if len(key_attributes["enum"]) == 1:
                print(
                    "WARNING: enum value with single choice for field {}, choice {}."
                    "".format(key_name, key_attributes["enum"])
                )
                continue

            # Max length find
            max_length = 255
            for choice in key_attributes["enum"]:
                if len(choice) > 255:
                    max_length = len(choice)

            choices = tuple(set(zip(key_attributes["enum"], key_attributes["enum"])))

            field_str = (
                "    {} = models.CharField(choices={}, max_length={}, "
                "default='{}', {})\n"
                "".format(
                    key_name,
                    choices,
                    max_length,
                    key_attributes["enum"][0],
                    required_str,
                )
            )

        # Date time field
        elif (
            key_attributes["type"] == "string"
            and key_attributes.get("format") == "date-time"
        ):
            auto_now_add = False
            editable = True
            if key_name in ["created_on", "modified_on"]:
                auto_now_add = True
                editable = False

            field_str = (
                "    {} = models.DateTimeField(auto_now_add={}, editable={}, {})\n"
                "".format(key_name, auto_now_add, editable, required_str)
            )

        elif key_attributes["type"] == "string":
            field_str = "    {} = models.TextField({})\n".format(key_name, required_str)

        elif key_attributes["type"] == "number":
            field_str = "    {} = models.IntegerField({})\n".format(
                key_name, required_str
            )

        elif key_attributes["type"] == "array":
            field_str = "    {} = json.JSONField(default=[], {})\n".format(
                key_name, required_str
            )

        elif key_attributes["type"] == "object":
            field_str = "    {} = json.JSONField(default={{}}, {})\n".format(
                key_name, required_str
            )

        elif key_attributes["type"] == "boolean":
            field_str = "    {} = models.BooleanField(default=False, {})\n".format(
                key_name, required_str
            )

        model_str += field_str

    model_str += "\n    class Meta:\n        db_table = '{}s'\n".format(
        model_name.lower()
    )

    print(model_str)


if __name__ == "__main__":
    parse_model()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['oebl_irs_workflow', 'oebl_irs_workflow.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.1.4,<4.0.0',
 'apis-core>=0.15.0,<0.16.0',
 'djangorestframework>=3.12.2,<4.0.0']

setup_kwargs = {
    'name': 'oebl-irs-workflow',
    'version': '0.1.2',
    'description': 'Workflow module for the IRS extension of the APIS framework',
    'long_description': None,
    'author': 'Matthias Schlögl',
    'author_email': 'm.schloegl@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<3.7',
}


setup(**setup_kwargs)
